@@ -1,7 +1,3 @@
-/*
- * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
- */
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -318,8 +314,12 @@ next_token(char *p, char *end, char **new_p, int *delim)
 			   ((*delim) && (!isdelim(*p))) ||
 			   (isdelim(*p))
 			   )
-			 {
+			 {/*the line below this is never  used because it skips to
+                           * the 'done' label which is after the return call for
+                           * in_tok being 0. is this intentional?
+                           */
 			    in_tok = 0;
+
 			    tok_end = p - 1;
 			    if (*p == '\n') line--;
 			    goto done;
@@ -445,7 +445,7 @@ parse(char *data, off_t size)
    p = data;
    end = data + size;
    line = 1;
-   while ((token = next_token(p, end, &p, &delim)) != NULL)
+   while ((token = next_token(p, end, &p, &delim)))
      {
 	/* if we are in param mode, the only delimiter
 	 * we'll accept is the semicolon
@@ -644,6 +644,8 @@ compile(void)
    int fd;
    off_t size;
    char *data, *p;
+   Eina_List *l;
+   Edje_Style *stl;
 
    if (!tmp_dir)
 #ifdef HAVE_EVIL
@@ -720,47 +722,20 @@ compile(void)
                     }
                }
           }
-	/*
-	 * On some BSD based systems (MacOS, OpenBSD), the default cpp
-	 * in the path is a wrapper script that chokes on the -o option.
-	 * If the preprocessor is invoked via gcc -E, it will treat
-	 * file_in as a linker file. The safest route seems to be to
-	 * run cpp with the output as the second non-option argument.
-	 *
-	 * Redirecting the output is required for MacOS 10.3, and works fine
-	 * on other systems.
-	 *
-	 * Also, the MacOS preprocessor is not managing C++ comments, so pass gcc
-	 * preprocessor just after. Linux gcc seems to not like it, so guard the
-	 * code so that it is compiled only on MacOS
-	 *
-	 */
+
+        /* Trying gcc and other syntax */
 	if (ret != 0)
 	  {
-	     snprintf(buf, sizeof(buf), "cat %s | cpp -I%s %s > %s",
-		      file_in, inc, def, tmpn);
+	     snprintf(buf, sizeof(buf), "%s -I%s %s -E -o %s -std=c99 - < %s",
+                      getenv("CC") ? getenv("CC") : "cc",
+		      inc, def, tmpn, file_in);
 	     ret = system(buf);
-#if defined (__MacOSX__) || ( defined (__MACH__) && defined (__APPLE__) ) || defined (__OpenBSD__)
-             if (ret == 0)
-               {
-                  static char tmpn2[4096];
-                  
-                  snprintf (tmpn2, PATH_MAX, "%s/edje_cc.edc-tmp-XXXXXX", tmp_dir);
-                  fd = mkstemp(tmpn2);
-                  if (fd >= 0)
-                    {
-                       close(fd); 
-                       snprintf (buf, 4096, "gcc -xc -I%s %s -E -o %s %s",
-                                 inc, def, tmpn2, tmpn);
-                       ret = system(buf);
-                       snprintf(tmpn, 4096, "%s", tmpn2);
-                    }
-               }
-#endif
 	  }
+        /* Trying suncc syntax */
 	if (ret != 0)
 	  {
-	     snprintf(buf, sizeof(buf), "gcc -I%s %s -E -o %s %s",
+	     snprintf(buf, sizeof(buf), "%s -I%s %s -E -o %s -xc99 - < %s",
+                      getenv("CC") ? getenv("CC") : "cc",
 		      inc, def, tmpn, file_in);
 	     ret = system(buf);
 	  }
@@ -807,6 +782,13 @@ compile(void)
      }
    free(data);
    close(fd);
+
+   EINA_LIST_FOREACH(edje_file->styles, l, stl)
+      if (!stl->name)
+	{
+	   ERR("%s: Error. style must have a name.", progname);
+	   exit(-1);
+	}
 }
 
 int
@@ -824,8 +806,8 @@ is_num(int n)
 {
    char *str;
    char *end;
-   long val;
-
+   long int ret;
+   
    str = eina_list_nth(params, n);
    if (!str)
      {
@@ -835,7 +817,11 @@ is_num(int n)
      }
    if (str[0] == 0) return 0;
    end = str;
-   val = strtol(str, &end, 0);
+   ret = strtol(str, &end, 0);
+   if ((ret == LONG_MIN) || (ret == LONG_MAX))
+     {
+        n = 0; // do nothing. shut gcc warnings up
+     }
    if ((end != str) && (end[0] == 0)) return 1;
    return 0;
 }
@@ -1246,12 +1232,12 @@ _is_numi(char c)
 int
 _is_op1i(char c)
 {
-   switch(c)
+   switch (c)
      {
      case '*':;
      case '%':;
      case '/': return 1;
-     default: return 0;
+     default: break;
      }
    return 0;
 }
@@ -1259,11 +1245,11 @@ _is_op1i(char c)
 int
 _is_op2i(char c)
 {
-   switch(c)
+   switch (c)
      {
      case '+':;
      case '-': return 1;
-     default: return 0;
+     default: break;
      }
    return 0;
 }
@@ -1297,8 +1283,8 @@ _calci(char op, int a, int b)
      default:
 	ERR("%s: Error. %s:%i unexpected character '%c'\n",
 	    progname, file_in, line - 1, op);
-	return a;
      }
+   return a;
 }
 
 /* float set of functoins */
@@ -1336,7 +1322,6 @@ _deltaf(char *s, double *val)
 	s++;
 	s = _alphaf(s, val);
 	s++;
-	return s;
      }
    return s;
 }
@@ -1454,8 +1439,7 @@ _is_numf(char c)
        || ('.' == c)
        || ('+' == c))
      return 1;
-   else
-     return 0;
+   return 0;
 }
 
 static int
@@ -1466,7 +1450,7 @@ _is_op1f(char c)
      case '*':;
      case '%':;
      case '/': return 1;
-     default: return 0;
+     default: break;
      }
    return 0;
 }
@@ -1478,7 +1462,7 @@ _is_op2f(char c)
      {
      case '+':;
      case '-': return 1;
-     default: return 0;
+     default: break;
      }
    return 0;
 }
@@ -1512,8 +1496,8 @@ _calcf(char op, double a, double b)
      default:
 	ERR("%s: Error. %s:%i unexpected character '%c'\n",
 	    progname, file_in, line - 1, op);
-	return a;
      }
+   return a;
 }
 
 static int
