@@ -1,7 +1,3 @@
-/*
- * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
- */
-
 /* ugly ugly. avert your eyes. */
 
 #ifdef HAVE_CONFIG_H
@@ -14,6 +10,7 @@
 #include <locale.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <errno.h>
 
 
 #include <Ecore_File.h>
@@ -21,6 +18,7 @@
 
 #include "edje_decc.h"
 
+int _edje_cc_log_dom = -1;
 char *progname = NULL;
 char *file_in = NULL;
 char *file_out = NULL;
@@ -45,6 +43,10 @@ main_help(void)
      ("Usage:\n"
       "\t%s input_file.edj [-main-out file.edc] [-no-build-sh] [-current-dir]\n"
       "\n"
+      " -main-out\tCreate a symbolic link to the main edc \n"
+      " -no-build-sh\tDon't output build.sh \n"
+      " -current-dir\tOutput to current directory \n"
+      "\n"
       ,progname);
 }
 
@@ -57,10 +59,25 @@ main(int argc, char **argv)
    int i;
 
    setlocale(LC_NUMERIC, "C");
-
+   if (!eina_init())
+     exit(-1);
+   _edje_cc_log_dom = eina_log_domain_register
+     ("edje_decc", EDJE_CC_DEFAULT_LOG_COLOR);
+   if (_edje_cc_log_dom < 0)
+     {
+       EINA_LOG_ERR("Impossible to create a log domain.");
+       eina_shutdown();
+       exit(-1);
+     }
+   eina_log_level_set(EINA_LOG_LEVEL_INFO);
    progname = argv[0];
    for (i = 1; i < argc; i++)
      {
+	if (!strcmp(argv[i], "-h"))
+	  {
+	     main_help();
+	     exit(0);
+	  }
 	if (!file_in)
 	  file_in = argv[i];
 	else if ((!strcmp(argv[i], "-main-out")) && (i < (argc - 1)))
@@ -75,20 +92,23 @@ main(int argc, char **argv)
      }
    if (!file_in)
      {
-	fprintf(stderr, "%s: Error: no input file specified.\n", progname);
+	ERR("%s: Error: no input file specified.", progname);
 	main_help();
 	exit(-1);
      }
 
-   edje_init();
-   eet_init();
+   if (!edje_init())
+     exit(-1);
    source_edd();
 
    if (!decomp()) return -1;
    output();
 
    eet_close(ef);
-   eet_shutdown();
+   edje_shutdown();
+   eina_log_domain_unregister(_edje_cc_log_dom);
+   _edje_cc_log_dom = -1;
+   eina_shutdown();
    return 0;
 }
 
@@ -98,27 +118,27 @@ decomp(void)
    ef = eet_open(file_in, EET_FILE_MODE_READ);
    if (!ef)
      {
-	printf("ERROR: cannot open %s\n", file_in);
+	ERR("ERROR: cannot open %s", file_in);
 	return 0;
      }
 
    srcfiles = source_load(ef);
    if (!srcfiles || !srcfiles->list)
      {
-	printf("ERROR: %s has no decompile information\n", file_in);
+	ERR("ERROR: %s has no decompile information", file_in);
 	eet_close(ef);
 	return 0;
      }
    if (!eina_list_data_get(srcfiles->list) || !root_filename_is_sane())
      {
-        printf("ERROR: Invalid root filename: '%s'\n", (char *) eina_list_data_get(srcfiles->list));
+        ERR("ERROR: Invalid root filename: '%s'", (char *) eina_list_data_get(srcfiles->list));
 	eet_close(ef);
 	return 0;
      }
-   edje_file = eet_data_read(ef, _edje_edd_edje_file, "edje_file");
+   edje_file = eet_data_read(ef, _edje_edd_edje_file, "edje/file");
    if (!edje_file)
      {
-	printf("ERROR: %s does not appear to be an edje file\n", file_in);
+        ERR("ERROR: %s does not appear to be an edje file", file_in);
 	eet_close(ef);
 	return 0;
      }
@@ -128,7 +148,7 @@ decomp(void)
      }
    else if (!compiler_cmd_is_sane())
      {
-	printf("ERROR: invalid compiler executable: '%s'\n", edje_file->compiler);
+	ERR("ERROR: invalid compiler executable: '%s'", edje_file->compiler);
 	eet_close(ef);
 	return 0;
      }
@@ -164,9 +184,12 @@ output(void)
    if (edje_file->image_dir)
      {
         Edje_Image_Directory_Entry *ei;
+	unsigned int i;
 
-	EINA_LIST_FOREACH(edje_file->image_dir->entries, l, ei)
+	for (i = 0; i < edje_file->image_dir->entries_count; ++i)
 	  {
+	     ei = &edje_file->image_dir->entries[i];
+
 	     if ((ei->source_type > EDJE_IMAGE_SOURCE_TYPE_NONE) &&
 		 (ei->source_type < EDJE_IMAGE_SOURCE_TYPE_LAST) &&
 		 (ei->source_type != EDJE_IMAGE_SOURCE_TYPE_EXTERNAL) &&
@@ -184,17 +207,17 @@ output(void)
 		  ee = ecore_evas_buffer_new(1, 1);
 		  if (!ee)
 		    {
-		       fprintf(stderr, "Error. cannot create buffer engine canvas for image save.\n");
+		       ERR("Cannot create buffer engine canvas for image save.");
 		       exit(-1);
 		    }
 		  evas = ecore_evas_get(ee);
 		  im = evas_object_image_add(evas);
 		  if (!im)
 		    {
-		       fprintf(stderr, "Error. cannot create image object for save.\n");
+		       ERR("Cannot create image object for save.");
 		       exit(-1);
 		    }
-		  snprintf(buf, sizeof(buf), "images/%i", ei->id);
+		  snprintf(buf, sizeof(buf), "edje/images/%i", ei->id);
 		  evas_object_image_file_set(im, file_in, buf);
 		  snprintf(out, sizeof(out), "%s/%s", outdir, ei->entry);
 		  printf("Output Image: %s\n", out);
@@ -203,14 +226,14 @@ output(void)
 		  *p = 0;
 		  if (strstr(pp, "../"))
 		    {
-		       printf("ERROR: potential security violation. attempt to write in parent dir.\n");
+		       ERR("Potential security violation. attempt to write in parent dir.");
 		       exit(-1);
 		    }
 		  ecore_file_mkpath(pp);
 		  free(pp);
 		  if (!evas_object_image_save(im, out, NULL, "quality=100 compress=9"))
 		    {
-		       printf("ERROR: cannot write file %s. Perhaps missing JPEG or PNG saver modules for Evas.\n", out);
+		       ERR("Cannot write file %s. Perhaps missing JPEG or PNG saver modules for Evas.", out);
 		       exit(-1);
 		    }
 		  evas_object_del(im);
@@ -228,26 +251,26 @@ output(void)
 	char *pp;
 
 	snprintf(out, sizeof(out), "%s/%s", outdir, sf->name);
-	printf("Output Source File: %s\n", out);
+	INF("Output Source File: %s\n", out);
 	pp = strdup(out);
 	p = strrchr(pp, '/');
 	*p = 0;
 	if (strstr(pp, "../"))
 	  {
-	     printf("ERROR: potential security violation. attempt to write in parent dir.\n");
+	     ERR("Potential security violation. attempt to write in parent dir.");
 	     exit (-1);
 	  }
 	ecore_file_mkpath(pp);
 	free(pp);
 	if (strstr(out, "../"))
 	  {
-	     printf("ERROR: potential security violation. attempt to write in parent dir.\n");
+	     ERR("Potential security violation. attempt to write in parent dir.");
 	     exit (-1);
 	  }
 	f = fopen(out, "wb");
 	if (!f)
 	  {
-	     printf("ERROR: unable to write file (%s).\n", out);
+	     ERR("Unable to write file (%s).", out);
 	     exit (-1);
 	  }
 
@@ -257,46 +280,56 @@ output(void)
 	if (sf->file) fputs(sf->file, f);
 	fclose(f);
      }
-   if (fontlist)
+   if (edje_file->fonts)
      {
-        Font *fn;
+        Edje_Font_Directory_Entry *fn;
+        Eina_Iterator *it;
 
-	EINA_LIST_FOREACH(fontlist->list, l, fn)
+        it = eina_hash_iterator_data_new(edje_file->fonts);
+	EINA_ITERATOR_FOREACH(it, fn)
 	  {
 	     void *font;
 	     int fontsize;
 	     char out[4096];
-
-	     snprintf(out, sizeof(out), "fonts/%s", fn->name);
+             /* FIXME!!!! */
+                                         /* should be fn->entry -v */
+	     snprintf(out, sizeof(out), "edje/fonts/%s", fn->file);
 	     font = eet_read(ef, out, &fontsize);
 	     if (font)
 	       {
 		  FILE *f;
 		  char *pp;
 
-		  snprintf(out, sizeof(out), "%s/%s", outdir, fn->file);
-		  printf("Output Font: %s\n", out);
+                                         /* should be fn->file -v */
+		  snprintf(out, sizeof(out), "%s/%s", outdir, fn->entry);
+		  INF("Output Font: %s", out);
 		  pp = strdup(out);
 		  p = strrchr(pp, '/');
 		  *p = 0;
 		  if (strstr(pp, "../"))
 		    {
-		       printf("ERROR: potential security violation. attempt to write in parent dir.\n");
+		       ERR("Potential security violation. attempt to write in parent dir.");
 		       exit (-1);
 		    }
 		  ecore_file_mkpath(pp);
 		  free(pp);
 		  if (strstr(out, "../"))
 		    {
-		       printf("ERROR: potential security violation. attempt to write in parent dir.\n");
+		       ERR("Potential security violation. attempt to write in parent dir.");
 		       exit (-1);
 		    }
-		  f = fopen(out, "wb");
-		  fwrite(font, fontsize, 1, f);
-		  fclose(f);
+		  if (!(f = fopen(out, "wb")))
+                    {
+                       ERR("Could not open file: %s", out);
+		       exit (-1);
+                    }
+		  if (fwrite(font, fontsize, 1, f) != 1)
+		    ERR("Could not write font: %s", strerror(errno));
+		  if (f) fclose(f);
 		  free(font);
 	       }
 	  }
+        eina_iterator_free(it);
      }
      {
 	char out[4096];
@@ -310,7 +343,7 @@ output(void)
 	     printf("Output Build Script: %s\n", out);
 	     if (strstr(out, "../"))
 	       {
-		  printf("ERROR: potential security violation. attempt to write in parent dir.\n");
+		  ERR("potential security violation. attempt to write in parent dir.\n");
 		  exit (-1);
 	       }
 	     f = fopen(out, "wb");
@@ -318,15 +351,18 @@ output(void)
 	     fprintf(f, "%s $@ -id . -fd . %s -o %s.edj\n", edje_file->compiler, sf->name, outdir);
 	     fclose(f);
 
-	     printf("\n*** CAUTION ***\n"
-		    "Please check the build script for anything malicious "
-		    "before running it!\n\n");
+	     WRN("\n*** CAUTION ***\n"
+		 "Please check the build script for anything malicious "
+		 "before running it!\n\n");
 	  }
 
 	if (file_out)
 	  {
 	     snprintf(out, sizeof(out), "%s/%s", outdir, file_out);
-	     symlink(sf->name, out);
+	     if (symlink(sf->name, out) != 0)
+               {
+                  ERR("symlink %s -> %s failed\n", sf->name, out);
+               }
 	  }
 
 	chmod(out, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IWGRP | S_IXGRP);
@@ -338,9 +374,9 @@ output(void)
 static int
 compiler_cmd_is_sane()
 {
-   char *c = edje_file->compiler, *ptr;
+   const char *c = edje_file->compiler, *ptr;
 
-   if (!c || !*c)
+   if ((!c) || (!*c))
      {
 	return 0;
      }
@@ -348,7 +384,7 @@ compiler_cmd_is_sane()
    for (ptr = c; ptr && *ptr; ptr++)
      {
 	/* only allow [a-z][A-Z][0-9]_- */
-	if (!isalnum(*ptr) && *ptr != '_' && *ptr != '-')
+	if ((!isalnum(*ptr)) && (*ptr != '_') && (*ptr != '-'))
 	  {
 	     return 0;
 	  }
