@@ -1,11 +1,7 @@
-/*
- * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
- */
-
 #include "edje_private.h"
 
 static void
-_edje_hold_signal_cb(void *data, Evas * e, Evas_Object * obj, void *event_info)
+_edje_hold_signal_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj, void *event_info)
 {
    Evas_Event_Hold *ev;
    Edje *ed;
@@ -19,12 +15,10 @@ _edje_hold_signal_cb(void *data, Evas * e, Evas_Object * obj, void *event_info)
      _edje_emit(ed, "hold,on", rp->part->name);
    else
      _edje_emit(ed, "hold,off", rp->part->name);
-   return;
-   e = NULL;
 }
 
 static void
-_edje_focus_in_signal_cb(void *data, Evas * e, Evas_Object * obj, __UNUSED__ void *event_info)
+_edje_focus_in_signal_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Edje *ed;
    Edje_Real_Part *rp;
@@ -35,12 +29,10 @@ _edje_focus_in_signal_cb(void *data, Evas * e, Evas_Object * obj, __UNUSED__ voi
      return;
 
    _edje_emit(ed, "focus,part,in", rp->part->name);
-   return;
-   e = NULL;
 }
 
 static void
-_edje_focus_out_signal_cb(void *data, Evas * e, Evas_Object * obj, __UNUSED__ void *event_info)
+_edje_focus_out_signal_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Edje *ed;
    Edje_Real_Part *rp;
@@ -51,12 +43,10 @@ _edje_focus_out_signal_cb(void *data, Evas * e, Evas_Object * obj, __UNUSED__ vo
      return;
 
    _edje_emit(ed, "focus,part,out", rp->part->name);
-   return;
-   e = NULL;
 }
 
 static void
-_edje_mouse_in_signal_cb(void *data, Evas * e, Evas_Object * obj, void *event_info)
+_edje_mouse_in_signal_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj, void *event_info)
 {
    Evas_Event_Mouse_In *ev;
    Edje *ed;
@@ -69,12 +59,10 @@ _edje_mouse_in_signal_cb(void *data, Evas * e, Evas_Object * obj, void *event_in
        ((ev->event_flags) &&
 	(rp->part->ignore_flags & ev->event_flags))) return;
    _edje_emit(ed, "mouse,in", rp->part->name);
-   return;
-   e = NULL;
 }
 
 static void
-_edje_mouse_out_signal_cb(void *data, Evas * e, Evas_Object * obj, void *event_info)
+_edje_mouse_out_signal_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj, void *event_info)
 {
    Evas_Event_Mouse_Out *ev;
    Edje *ed;
@@ -87,12 +75,10 @@ _edje_mouse_out_signal_cb(void *data, Evas * e, Evas_Object * obj, void *event_i
        ((ev->event_flags) &&
 	(rp->part->ignore_flags & ev->event_flags))) return;
    _edje_emit(ed, "mouse,out", rp->part->name);
-   return;
-   e = NULL;
 }
 
 static void
-_edje_mouse_down_signal_cb(void *data, Evas * e, Evas_Object * obj, void *event_info)
+_edje_mouse_down_signal_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj, void *event_info)
 {
    Evas_Event_Mouse_Down *ev;
    Edje *ed;
@@ -198,17 +184,16 @@ _edje_mouse_down_signal_cb(void *data, Evas * e, Evas_Object * obj, void *event_
    if (rp->clicked_button == 0)
      {
 	rp->clicked_button = ev->button;
-	rp->still_in = 1;
+        if (!(ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD))
+         rp->still_in = 1;
      }
 //   _edje_recalc_do(ed);
    _edje_thaw(ed);
    _edje_unref(ed);
-   return;
-   e = NULL;
 }
 
 static void
-_edje_mouse_up_signal_cb(void *data, Evas * e, Evas_Object * obj, void *event_info)
+_edje_mouse_up_signal_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj, void *event_info)
 {
    Evas_Event_Mouse_Up *ev;
    Edje *ed;
@@ -271,12 +256,10 @@ _edje_mouse_up_signal_cb(void *data, Evas * e, Evas_Object * obj, void *event_in
 //   _edje_recalc_do(ed);
    _edje_thaw(ed);
    _edje_unref(ed);
-   return;
-   e = NULL;
 }
 
 static void
-_edje_mouse_move_signal_cb(void *data, Evas * e, Evas_Object * obj, void *event_info)
+_edje_mouse_move_signal_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj, void *event_info)
 {
    Evas_Event_Mouse_Move *ev;
    Edje *ed;
@@ -297,21 +280,30 @@ _edje_mouse_move_signal_cb(void *data, Evas * e, Evas_Object * obj, void *event_
 
    if (rp->still_in)
      {
-	Evas_Coord x, y, w, h;
 
-	evas_object_geometry_get(obj, &x, &y, &w, &h);
-	if ((ev->cur.canvas.x < x) || (ev->cur.canvas.y < y) ||
-	    (ev->cur.canvas.x >= (x + w)) || (ev->cur.canvas.y >= (y + h)))
-	  rp->still_in = 0;
+        if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD)
+          rp->still_in = 0;
+        else
+          {
+            Evas_Coord x, y, w, h;
+            
+            evas_object_geometry_get(obj, &x, &y, &w, &h);
+            if ((ev->cur.canvas.x < x) || (ev->cur.canvas.y < y) ||
+                (ev->cur.canvas.x >= (x + w)) || (ev->cur.canvas.y >= (y + h)))
+              rp->still_in = 0;
+          }
      }
    else
      {
-	Evas_Coord x, y, w, h;
+        if (!(ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD))
+          {
+            Evas_Coord x, y, w, h;
 
-	evas_object_geometry_get(obj, &x, &y, &w, &h);
-	if ((ev->cur.canvas.x >= x) && (ev->cur.canvas.y >= y) &&
-	    (ev->cur.canvas.x < (x + w)) && (ev->cur.canvas.y < (y + h)))
-	  rp->still_in = 1;
+            evas_object_geometry_get(obj, &x, &y, &w, &h);
+            if ((ev->cur.canvas.x >= x) && (ev->cur.canvas.y >= y) &&
+                (ev->cur.canvas.x < (x + w)) && (ev->cur.canvas.y < (y + h)))
+              rp->still_in = 1;
+          }
      }
    _edje_freeze(ed);
    if (rp->drag)
@@ -350,12 +342,10 @@ _edje_mouse_move_signal_cb(void *data, Evas * e, Evas_Object * obj, void *event_
      }
    _edje_unref(ed);
    _edje_thaw(ed);
-   return;
-   e = NULL;
 }
 
 static void
-_edje_mouse_wheel_signal_cb(void *data, Evas * e, Evas_Object * obj, void *event_info)
+_edje_mouse_wheel_signal_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj, void *event_info)
 {
    Evas_Event_Mouse_Wheel *ev;
    Edje *ed;
@@ -371,12 +361,10 @@ _edje_mouse_wheel_signal_cb(void *data, Evas * e, Evas_Object * obj, void *event
 
    snprintf(buf, sizeof(buf), "mouse,wheel,%i,%i", ev->direction, (ev->z < 0) ? (-1) : (1));
    _edje_emit(ed, buf, rp->part->name);
-   return;
-   e = NULL;
 }
 
 Eina_Bool
-_edje_timer_cb(__UNUSED__ void *data)
+_edje_timer_cb(void *data __UNUSED__)
 {
    double t;
    Eina_List *l;
