@@ -1,7 +1,3 @@
-/*
- * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
- */
-
 #include "edje_private.h"
 
 static Eina_Hash *type_registry = NULL;
@@ -129,7 +125,7 @@ edje_object_part_external_param_set(Evas_Object *obj, const char *part, const Ed
 	return EINA_FALSE;
      }
 
-   return _edje_external_param_set(rp->swallowed_object, param);
+   return _edje_external_param_set(obj, rp, param);
 }
 
 /**
@@ -178,7 +174,30 @@ edje_object_part_external_param_get(const Evas_Object *obj, const char *part, Ed
 	return EINA_FALSE;
      }
 
-   return _edje_external_param_get(rp->swallowed_object, param);
+   return _edje_external_param_get(obj, rp, param);
+}
+
+
+
+EAPI Evas_Object*
+edje_object_part_external_content_get(const Evas_Object *obj, const char *part, const char *content)
+{
+   Edje *ed;
+   Edje_Real_Part *rp;
+
+   if (!content) return EINA_FALSE;
+
+   ed = _edje_fetch(obj);
+   if ((!ed) || (!part)) return EINA_FALSE;
+
+   rp = _edje_real_part_recursive_get(ed, (char *)part);
+   if (!rp)
+     {
+	ERR("no part '%s'", part);
+	return EINA_FALSE;
+     }
+
+   return _edje_external_content_get(rp->swallowed_object, content);
 }
 
 /**
@@ -220,7 +239,7 @@ edje_object_part_external_param_type_get(const Evas_Object *obj, const char *par
 	    type->module_name);
 	return EDJE_EXTERNAL_PARAM_TYPE_MAX;
      }
-   for (info = type->parameters_info; info->name != NULL; info++)
+   for (info = type->parameters_info; info->name; info++)
      if (strcmp(info->name, param) == 0)
        return info->type;
 
@@ -293,11 +312,11 @@ edje_external_type_unregister(const char *type_name)
  *
  * @note the given array is not modified, but the type name strings
  *       are @b not duplicated! That is, all type names must be @b
- *       live until they are unregistered! This was choosen to save
+ *       live until they are unregistered! This was chosen to save
  *       some memory and most people will just define the array as a
  *       global static const type anyway.
  *
- * @param arrray @c NULL terminated array with type name and
+ * @param array @c NULL terminated array with type name and
  *        information. Note that type name or information are not
  *        modified by are @b referenced, so they must keep alive after
  *        this function returns!
@@ -374,6 +393,46 @@ edje_external_type_abi_version_get(void)
    return EDJE_EXTERNAL_TYPE_ABI_VERSION;
 }
 
+/**
+ * Returns an iterator that emits Eina_Hash_Tuple pointers with key
+ * being the name and data being the Edje_External_Type pointer.
+ *
+ * @code
+ * const Eina_Hash_Tuple *tuple;
+ * Eina_Iterator *itr;
+ * const Eina_List *l, *modules;
+ * const char *s;
+ *
+ * modules = edje_available_modules_get();
+ * EINA_LIST_FOREACH(modules, l, s)
+ *   {
+ *      if (!edje_module_load(s))
+ *        printf("Error loading edje module: %s\n", s);
+ *   }
+ *
+ * itr = edje_external_iterator_get();
+ * EINA_ITERATOR_FOREACH(itr, tuple)
+ *   {
+ *      const char *name = tuple->key;
+ *      const Edje_External_Type *type = tuple->data;
+ *
+ *      if ((!type) ||
+ *          (type->abi_version != edje_external_type_abi_version_get()))
+ *        {
+ *           printf("Error: invalid type %p (abi: %d, expected: %d)\n",
+ *                   type, type ? type->abi_version : 0,
+ *                   edje_external_type_abi_version_get());
+ *           continue;
+ *        }
+ *
+ *      printf("%s: %s (%s) label='%s' desc='%s'\n",
+ *             name, type->module, type->module_name,
+ *             type->label_get ? type->label_get(type->data) : "",
+ *             type->description_get ? type->description_get(type->data) : "");
+ *   }
+ *
+ * @endcode
+ */
 EAPI Eina_Iterator *
 edje_external_iterator_get(void)
 {
@@ -566,12 +625,23 @@ _edje_external_signal_emit(Evas_Object *obj, const char *emission, const char *s
 }
 
 Eina_Bool
-_edje_external_param_set(Evas_Object *obj, const Edje_External_Param *param)
+_edje_external_param_set(Evas_Object *obj, Edje_Real_Part *rp, const Edje_External_Param *param)
 {
-   Edje_External_Type *type = evas_object_data_get(obj, "Edje_External_Type");
+   Evas_Object *swallowed_object = rp->swallowed_object;
+   Edje_External_Type *type = evas_object_data_get(swallowed_object, "Edje_External_Type");
    if (!type)
      {
-	ERR("no external type for object %p", obj);
+	if ((rp->part->type == EDJE_PART_TYPE_TEXT) ||
+            (rp->part->type == EDJE_PART_TYPE_TEXTBLOCK))
+	  {
+	     if ((param->type == EDJE_EXTERNAL_PARAM_TYPE_STRING) &&
+		 (!strcmp (param->name, "text")) && (obj))
+		{
+	           return edje_object_part_text_set(obj, rp->part->name, param->s);
+		}
+	  }
+
+	ERR("no external type for object %p", swallowed_object);
 	return EINA_FALSE;
      }
    if (!type->param_set)
@@ -580,11 +650,41 @@ _edje_external_param_set(Evas_Object *obj, const Edje_External_Param *param)
 	    type->module_name, type->module);
 	return EINA_FALSE;
      }
-   return type->param_set(type->data, obj, param);
+   return type->param_set(type->data, swallowed_object, param);
 }
 
 Eina_Bool
-_edje_external_param_get(const Evas_Object *obj, Edje_External_Param *param)
+_edje_external_param_get(const Evas_Object *obj, Edje_Real_Part *rp, Edje_External_Param *param)
+{
+   Evas_Object *swallowed_object = rp->swallowed_object;
+   Edje_External_Type *type = evas_object_data_get(swallowed_object, "Edje_External_Type");
+   if (!type)
+     {
+	if ((rp->part->type == EDJE_PART_TYPE_TEXT) ||
+            (rp->part->type == EDJE_PART_TYPE_TEXTBLOCK))
+	  {
+	     const char *text;
+	     param->type = EDJE_EXTERNAL_PARAM_TYPE_STRING;
+	     param->name = "text";
+	     text = edje_object_part_text_get(obj, rp->part->name);
+             param->s = text;
+	     return EINA_TRUE;
+	  }
+       
+	ERR("no external type for object %p", swallowed_object);
+	return EINA_FALSE;
+     }
+   if (!type->param_get)
+     {
+	ERR("external type '%s' from module '%s' does not provide param_get()",
+	    type->module_name, type->module);
+	return EINA_FALSE;
+     }
+   return type->param_get(type->data, swallowed_object, param);
+}
+
+Evas_Object*
+_edje_external_content_get(const Evas_Object *obj, const char *content)
 {
    Edje_External_Type *type = evas_object_data_get(obj, "Edje_External_Type");
    if (!type)
@@ -592,13 +692,13 @@ _edje_external_param_get(const Evas_Object *obj, Edje_External_Param *param)
 	ERR("no external type for object %p", obj);
 	return EINA_FALSE;
      }
-   if (!type->param_get)
+   if (!type->content_get)
      {
-	ERR("external type '%s' from module '%s' does not provide param_set()",
+	ERR("external type '%s' from module '%s' does not provide content_get()",
 	    type->module_name, type->module);
 	return EINA_FALSE;
      }
-   return type->param_get(type->data, obj, param);
+   return type->content_get(type->data, obj, content);
 }
 
 void
@@ -618,31 +718,34 @@ _edje_external_params_free(Eina_List *external_params, Eina_Bool free_strings)
 }
 
 void
-_edje_external_recalc_apply(Edje *ed, Edje_Real_Part *ep,
-			    Edje_Calc_Params *params,
-			    Edje_Part_Description *chosen_desc)
+_edje_external_recalc_apply(Edje *ed __UNUSED__, Edje_Real_Part *ep,
+			    Edje_Calc_Params *params __UNUSED__,
+			    Edje_Part_Description_Common *chosen_desc __UNUSED__)
 {
    Edje_External_Type *type;
+   Edje_Part_Description_External *ext;
    void *params1, *params2 = NULL;
-   if (!ep->swallowed_object) return;
 
+   if (!ep->swallowed_object) return;
    type = evas_object_data_get(ep->swallowed_object, "Edje_External_Type");
 
-   if (!type) return;
+   if ((!type) || (!type->state_set)) return;
 
-   if (!type->state_set) return;
+   ext = (Edje_Part_Description_External*) ep->param1.description;
 
    params1 = ep->param1.external_params ?
-		  ep->param1.external_params :
-		  ep->param1.description->external_params;
+     ep->param1.external_params : ext->external_params;
 
    if (ep->param2 && ep->param2->description)
-     params2 = ep->param2->external_params ?
-		  ep->param2->external_params :
-		  ep->param2->description->external_params;
+     {
+	ext = (Edje_Part_Description_External*) ep->param2->description;
+
+	params2 = ep->param2->external_params ?
+          ep->param2->external_params : ext->external_params;
+     }
 
    type->state_set(type->data, ep->swallowed_object,
-	 params1, params2, ep->description_pos);
+                   params1, params2, ep->description_pos);
 }
 
 void *
