@@ -1,9 +1,3 @@
-/*
- * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
- */
-
-#include <string.h>
-
 #include "edje_private.h"
 
 static int _injob = 0;
@@ -12,6 +6,8 @@ static Ecore_Timer *_job_loss_timer = NULL;
 
 static Eina_List *msgq = NULL;
 static Eina_List *tmp_msgq = NULL;
+static int tmp_msgq_processing = 0;
+static int tmp_msgq_restart = 0;
 
 /*============================================================================*
  *                                   API                                      *
@@ -49,7 +45,7 @@ EAPI void
 edje_object_message_send(Evas_Object *obj, Edje_Message_Type type, int id, void *msg)
 {
    Edje *ed;
-   int i;
+   unsigned int i;
 
    ed = _edje_fetch(obj);
    if (!ed) return;
@@ -77,7 +73,7 @@ edje_object_message_send(Evas_Object *obj, Edje_Message_Type type, int id, void
  */
 
 EAPI void
-edje_object_message_handler_set(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj, Edje_Message_Type type, int id, void *msg), void *data)
+edje_object_message_handler_set(Evas_Object *obj, Edje_Message_Handler_Cb func, void *data)
 {
    Edje *ed;
 
@@ -100,20 +96,25 @@ edje_object_message_handler_set(Evas_Object *obj, void (*func) (void *data, Evas
 EAPI void
 edje_object_message_signal_process(Evas_Object *obj)
 {
-   Eina_List *l, *tmpq = NULL;
+   Eina_List *l, *ln, *tmpq = NULL;
    Edje *ed;
    Edje_Message *em;
-   const void *data;
+   int gotos = 0;
 
    ed = _edje_fetch(obj);
    if (!ed) return;
 
-   EINA_LIST_FOREACH(msgq, l, em)
-     if (em->edje == ed)
-       tmpq = eina_list_append(tmpq, em);
-   /* now remove them from the old queue */
-   EINA_LIST_FOREACH(tmpq, l, data)
-     msgq = eina_list_remove(msgq, data);
+   for (l = msgq; l; )
+     {
+        ln = l->next;
+        em = l->data;
+        if (em->edje == ed)
+          {
+             tmpq = eina_list_append(tmpq, em);
+             msgq = eina_list_remove_list(msgq, l);
+          }
+        l = ln;
+     }
    /* a temporary message queue */
    if (tmp_msgq)
      {
@@ -129,16 +130,70 @@ edje_object_message_signal_process(Evas_Object *obj)
 	tmpq = NULL;
      }
 
+#if 0   
    while (tmp_msgq)
      {
 	Edje_Message *em;
 
 	em = tmp_msgq->data;
 	tmp_msgq = eina_list_remove_list(tmp_msgq, tmp_msgq);
-	em->edje->message.num--;
-	_edje_message_process(em);
-	_edje_message_free(em);
+        if (!ed->delete_me)
+          {
+             ed->processing_messages++;
+             _edje_message_process(em);
+             _edje_message_free(em);
+             ed->processing_messages--;
+          }
+        else
+           _edje_message_free(em);
      }
+#else
+   tmp_msgq_processing++;
+again:
+   EINA_LIST_FOREACH_SAFE(tmp_msgq, l, ln, em)
+     {
+        if (em->edje != ed) continue;
+	tmp_msgq = eina_list_remove_list(tmp_msgq, l);
+        if (!ed->delete_me)
+          {
+             ed->processing_messages++;
+             _edje_message_process(em);
+             _edje_message_free(em);
+             ed->processing_messages--;
+          }
+        else
+           _edje_message_free(em);
+        if (ed->processing_messages == 0)
+          {
+             if (ed->delete_me) _edje_del(ed);
+          }
+        // if some child callback in _edje_message_process called
+        // edje_object_message_signal_process() or
+        // edje_message_signal_process() then those will mark the restart
+        // flag when they finish - it mabsicammyt means tmp_msgq and
+        // any item in it has potentially become invalid - so that means l
+        // and ln could be rogue pointers, so start again from the beginning
+        // and skip anything that is not this object and process only what is.
+        // to avoid self-feeding loops allow a max of 1024 loops.
+        if (tmp_msgq_restart)
+          {
+             tmp_msgq_restart = 0;
+             gotos++;
+             if (gotos < 1024) goto again;
+             else
+               {
+                  WRN("Edje is in a self-feeding message loop (> 1024 gotos needed in a row)");
+                  goto end;
+               }
+          }
+     }
+end:
+   tmp_msgq_processing--;
+   if (tmp_msgq_processing == 0)
+      tmp_msgq_restart = 0;
+   else
+      tmp_msgq_restart = 1;
+#endif
 }
 
 /**
@@ -211,8 +266,19 @@ _edje_message_shutdown(void)
 void
 _edje_message_cb_set(Edje *ed, void (*func) (void *data, Evas_Object *obj, Edje_Message_Type type, int id, void *msg), void *data)
 {
+   unsigned int i;
+
    ed->message.func = func;
    ed->message.data = data;
+   for (i = 0 ; i < ed->table_parts_size ; i++) {
+      Edje_Real_Part *rp;
+      rp = ed->table_parts[i];
+      if (rp->part->type == EDJE_PART_TYPE_GROUP && rp->swallowed_object) {
+         Edje *edj2 = _edje_fetch(rp->swallowed_object);
+         if (!edj2) continue;
+	 _edje_message_cb_set(edj2, func, data);
+      }
+   }
 }
 
 Edje_Message *
@@ -633,7 +699,8 @@ _edje_message_process(Edje_Message *em)
 {
    Embryo_Function fn;
    void *pdata;
-
+   int ret;
+   
    /* signals are only handled one way */
    if (em->type == EDJE_MESSAGE_SIGNAL)
      {
@@ -674,7 +741,32 @@ _edje_message_process(Edje_Message *em)
    pdata = embryo_program_data_get(em->edje->collection->script);
    embryo_program_data_set(em->edje->collection->script, em->edje);
    embryo_program_max_cycle_run_set(em->edje->collection->script, 5000000);
-   embryo_program_run(em->edje->collection->script, fn);
+   ret = embryo_program_run(em->edje->collection->script, fn);
+   if (ret == EMBRYO_PROGRAM_FAIL)
+     {
+        ERR("ERROR with embryo script. "
+            "OBJECT NAME: '%s', "
+            "OBJECT FILE: '%s', "
+            "ENTRY POINT: '%s', "
+            "ERROR: '%s'",
+            em->edje->collection->part,
+            em->edje->file->path,
+            "message",
+            embryo_error_string_get(embryo_program_error_get(em->edje->collection->script)));
+     }
+   else if (ret == EMBRYO_PROGRAM_TOOLONG)
+     {
+        ERR("ERROR with embryo script. "
+            "OBJECT NAME: '%s', "
+            "OBJECT FILE: '%s', "
+            "ENTRY POINT: '%s', "
+            "ERROR: 'Script exceeded maximum allowed cycle count of %i'",
+            em->edje->collection->part,
+            em->edje->file->path,
+            "message",
+            embryo_program_max_cycle_run_get(em->edje->collection->script));
+     }
+   
    embryo_program_data_set(em->edje->collection->script, pdata);
    embryo_program_vm_pop(em->edje->collection->script);
 }
@@ -684,7 +776,7 @@ _edje_message_queue_process(void)
 {
    int i;
 
-   if (msgq == NULL) return;
+   if (!msgq) return;
 
    /* allow the message queue to feed itself up to 8 times before forcing */
    /* us to go back to normal processing and let a 0 timeout deal with it */
@@ -705,6 +797,7 @@ _edje_message_queue_process(void)
 	     msgq = NULL;
 	  }
 
+        tmp_msgq_processing++;
 	while (tmp_msgq)
 	  {
 	     Edje_Message *em;
@@ -728,12 +821,29 @@ _edje_message_queue_process(void)
 		  if (ed->delete_me) _edje_del(ed);
 	       }
 	  }
+        tmp_msgq_processing--;
+        if (tmp_msgq_processing == 0)
+           tmp_msgq_restart = 0;
+        else
+           tmp_msgq_restart = 1;
      }
 
    /* if the message queue filled again set a timer to expire in 0.0 sec */
    /* to get the idle enterer to be run again */
    if (msgq)
      {
+        static int self_feed_debug = -1;
+        
+        if (self_feed_debug == -1)
+          {
+             const char *s = getenv("EDJE_SELF_FEED_DEBUG");
+             if (s) self_feed_debug = atoi(s);
+             else self_feed_debug = 0;
+          }
+        if (self_feed_debug)
+          {
+             WRN("Edje is in a self-feeding message loop (> 8 loops needed)");
+          }
 	ecore_timer_add(0.0, _edje_dummy_timer, NULL);
      }
 }
