@@ -1,9 +1,3 @@
-/*
- * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
- */
-
-#include <string.h>
-
 #include "edje_private.h"
 
 
@@ -40,34 +34,46 @@ _edje_text_init(void)
 void
 _edje_text_part_on_add(Edje *ed, Edje_Real_Part *ep)
 {
-   Eina_List *tmp;
    Edje_Part *pt = ep->part;
-   Edje_Part_Description *desc;
+   Edje_Part_Description_Text *desc;
+   unsigned int i;
 
    if (ep->part->type != EDJE_PART_TYPE_TEXT) return;
 
    /* if text class exists for this part, add the edje to the tc member list */
-   if ((pt->default_desc) && (pt->default_desc->text.text_class))
-     _edje_text_class_member_add(ed, pt->default_desc->text.text_class);
+   desc = (Edje_Part_Description_Text *) pt->default_desc;
+   if ((pt->default_desc) && (desc->text.text_class))
+     _edje_text_class_member_add(ed, desc->text.text_class);
 
    /* If any other classes exist add them */
-   EINA_LIST_FOREACH(pt->other_desc, tmp, desc)
-     if ((desc) && (desc->text.text_class))
-       _edje_text_class_member_add(ed, desc->text.text_class);
+   for (i = 0; i < pt->other.desc_count; ++i)
+     {
+	desc = (Edje_Part_Description_Text *) pt->other.desc[i];
+	if ((desc) && (desc->text.text_class))
+	  _edje_text_class_member_add(ed, desc->text.text_class);
+     }
 }
 
 void
 _edje_text_part_on_del(Edje *ed, Edje_Part *pt)
 {
-   Eina_List *tmp;
-   Edje_Part_Description *desc;
+   Edje_Part_Description_Text *desc;
+   unsigned int i;
 
-   if ((pt->default_desc) && (pt->default_desc->text.text_class))
-     _edje_text_class_member_del(ed, pt->default_desc->text.text_class);
+   if (pt->type != EDJE_PART_TYPE_TEXT
+       && pt->type != EDJE_PART_TYPE_TEXTBLOCK)
+     return ;
 
-   EINA_LIST_FOREACH(pt->other_desc, tmp, desc)
-     if (desc->text.text_class)
-       _edje_text_class_member_del(ed, desc->text.text_class);
+   desc = (Edje_Part_Description_Text *) pt->default_desc;
+   if ((pt->default_desc) && (desc->text.text_class))
+     _edje_text_class_member_del(ed, desc->text.text_class);
+
+   for (i = 0; i < pt->other.desc_count; ++i)
+     {
+	desc = (Edje_Part_Description_Text *) pt->other.desc[i];
+	if (desc->text.text_class)
+	  _edje_text_class_member_del(ed, desc->text.text_class);
+     }
 }
 
 static void
@@ -111,8 +117,10 @@ _edje_text_fit_x(Edje *ed, Edje_Real_Part *ep,
 {
    Evas_Coord tw = 0, th = 0, p;
    int l, r;
+   int i;
    char *buf;
-   int c1 = -1, c2 = -1, loop = 0, extra;
+   int uc1 = -1, uc2 = -1, c1 = -1, c2 = -1;
+   int loop = 0, extra;
    size_t orig_len;
    FLOAT_T sc;
 
@@ -136,23 +144,23 @@ _edje_text_fit_x(Edje *ed, Edje_Real_Part *ep,
      {
 	if (params->type.text.elipsis != 0.0)
           /* should be the last in text! not the rightmost */
-          c1 = evas_object_text_last_up_to_pos(ep->object,
+          uc1 = evas_object_text_last_up_to_pos(ep->object,
                 -p + l, th / 2);
 	if (params->type.text.elipsis != 1.0)
           /* should be the last in text! not the rightmost */
-          c2 = evas_object_text_last_up_to_pos(ep->object,
+          uc2 = evas_object_text_last_up_to_pos(ep->object,
                 -p + sw - r, th / 2);
-	if ((c1 < 0) && (c2 < 0))
+	if ((uc1 < 0) && (uc2 < 0))
 	  {
-	     c1 = 0;
-	     c2 = 0;
+	     uc1 = 0;
+	     uc2 = 0;
 	  }
      }
 
-   if (!(((c1 >= 0) || (c2 >= 0)) && (tw > sw)))
+   if (!(((uc1 >= 0) || (uc2 >= 0)) && (tw > sw)))
      return text;
 
-   if ((c1 == 0) && (c2 == 0))
+   if ((uc1 == 0) && (uc2 == 0))
      return text;
 
    orig_len = strlen(text);
@@ -166,6 +174,25 @@ _edje_text_fit_x(Edje *ed, Edje_Real_Part *ep,
    if (!(buf = malloc(orig_len + extra)))
      return text;
 
+   /* Convert uc1, uc2 -> c1, c2 */
+   i = 0;
+   if (uc1 > 0)
+     {
+        c1 = 0;
+        for ( ; i < uc1 ; i++)
+          {
+             c1 = evas_string_char_next_get(text, c1, NULL);
+          }
+     }
+   if (uc2 > 0)
+     {
+        c2 = c1;
+        for ( ; i < uc2 ; i++)
+          {
+             c2 = evas_string_char_next_get(text, c2, NULL);
+          }
+     }
+
    while (((c1 >= 0) || (c2 >= 0)) && (tw > sw))
      {
 	loop++;
@@ -239,7 +266,7 @@ static const char *
 _edje_text_font_get(const char *base, const char *new, char **free_later)
 {
    const char *base_style, *new_style, *aux;
-   int font_len, style_len;
+   size_t font_len, style_len;
 
    if (base && (!new))
      return base;
@@ -256,7 +283,7 @@ _edje_text_font_get(const char *base, const char *new, char **free_later)
 
    font_len = strlen(new);
    aux = strchr(base_style, ',');
-   style_len = (aux) ? (aux - base_style) : (int) strlen(base_style);
+   style_len = (aux) ? (size_t)(aux - base_style) : strlen(base_style);
 
    *free_later = malloc(font_len + style_len + 1);
    memcpy(*free_later, new, font_len);
@@ -267,12 +294,12 @@ _edje_text_font_get(const char *base, const char *new, char **free_later)
 }
 
 const char *
-_edje_text_class_font_get(Edje *ed, Edje_Part_Description *chosen_desc, int *size, char **free_later)
+_edje_text_class_font_get(Edje *ed, Edje_Part_Description_Text *chosen_desc, int *size, char **free_later)
 {
    Edje_Text_Class *tc;
    const char *text_class_name, *font;
 
-   font = chosen_desc->text.font;
+   font = edje_string_get(&chosen_desc->text.font);
    *size = chosen_desc->text.size;
 
    text_class_name = chosen_desc->text.text_class;
@@ -283,7 +310,7 @@ _edje_text_class_font_get(Edje *ed, Edje_Part_Description *chosen_desc, int *siz
    if (!tc)
      return font;
 
-   font = _edje_text_font_get(chosen_desc->text.font, tc->font, free_later);
+   font = _edje_text_font_get(edje_string_get(&chosen_desc->text.font), tc->font, free_later);
    *size = _edje_text_size_calc(*size, tc);
 
    return font;
@@ -292,9 +319,9 @@ _edje_text_class_font_get(Edje *ed, Edje_Part_Description *chosen_desc, int *siz
 void
 _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
 			Edje_Calc_Params *params,
-			Edje_Part_Description *chosen_desc)
+			Edje_Part_Description_Text *chosen_desc)
 {
-   const char	*text;
+   const char	*text, *str;
    const char	*font;
    char		*font2 = NULL;
    char         *sfont = NULL;
@@ -306,7 +333,7 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
 
    sc = ed->scale;
    if (sc == 0.0) sc = _edje_scale;
-   text = chosen_desc->text.text;
+   text = edje_string_get(&chosen_desc->text.text);
    font = _edje_text_class_font_get(ed, chosen_desc, &size, &sfont);
 
    if (ep->text.text) text = (char *) ep->text.text;
@@ -315,13 +342,13 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
 
    if (ep->text.text_source)
      {
-	text = ep->text.text_source->chosen_description->text.text;
+	text = edje_string_get(&(((Edje_Part_Description_Text *)ep->text.text_source->chosen_description)->text.text));
 	if (ep->text.text_source->text.text) text = ep->text.text_source->text.text;
      }
    if (ep->text.source)
      {
-	font = ep->text.source->chosen_description->text.font;
-	size = ep->text.source->chosen_description->text.size;
+	font = edje_string_get(&(((Edje_Part_Description_Text *)ep->text.source->chosen_description)->text.font));
+	size = ((Edje_Part_Description_Text *)ep->text.source->chosen_description)->text.size;
 	if (ep->text.source->text.font) font = ep->text.source->text.font;
 	if (ep->text.source->text.size > 0) size = ep->text.source->text.size;
      }
@@ -330,13 +357,18 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
    if (!font) font = "";
 
    /* check if the font is embedded in the .eet */
-   if (ed->file->font_hash)
+   if (ed->file->fonts)
      {
-	Edje_Font_Directory_Entry *fnt = eina_hash_find(ed->file->font_hash, font);
+	Edje_Font_Directory_Entry *fnt = eina_hash_find(ed->file->fonts, font);
 
 	if (fnt)
 	  {
-	     font = fnt->path;
+             char *font2;
+             
+             size_t len = strlen(font) + sizeof("edje/fonts/") + 1;
+             font2 = alloca(len);
+             sprintf(font2, "edje/fonts/%s", font);
+             font = font2;
 	     inlined_font = 1;
 	  }
      }
@@ -384,7 +416,7 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
    if (ep->text.cache.in_str) eina_stringshare_del(ep->text.cache.in_str);
    ep->text.cache.in_str = eina_stringshare_add(text);
    ep->text.cache.in_size = size;
-   if (chosen_desc->text.fit_x)
+   if (chosen_desc->text.fit_x && (ep->text.cache.in_str && eina_stringshare_strlen(ep->text.cache.in_str) > 0))
      {
         if (inlined_font) evas_object_text_font_source_set(ep->object, ed->path);
 	else evas_object_text_font_source_set(ep->object, NULL);
@@ -395,12 +427,9 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
 	part_get_geometry(ep, &tw, &th);
 	if (tw > sw)
 	  {
-	     int psize;
-
-	     psize = size;
 	     while ((tw > sw) && (size > 0) && (tw != 0))
 	       {
-		  psize = size;
+		  int psize = size;
 		  size = (size * sw) / tw;
 		  if ((psize - size) <= 0) size = psize - 1;
 		  if (inlined_font) evas_object_text_font_source_set(ep->object, ed->path);
@@ -414,13 +443,11 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
 	  }
 	else if (tw < sw)
 	  {
-	     int psize;
-
-	     psize = size;
 	     while ((tw < sw) && (size > 0) && (tw != 0))
 	       {
-		  psize = size;
+		  int psize = size;
 		  size = (size * sw) / tw;
+		  /* fprintf(stderr, "size = %i (%i, %i)\n", size, sw, tw); */
 		  if ((psize - size) >= 0) size = psize + 1;
 		  if (inlined_font) evas_object_text_font_source_set(ep->object, ed->path);
 		  else evas_object_text_font_source_set(ep->object, NULL);
@@ -432,7 +459,7 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
 	       }
 	  }
      }
-   if (chosen_desc->text.fit_y)
+   if (chosen_desc->text.fit_y && (ep->text.cache.in_str && eina_stringshare_strlen(ep->text.cache.in_str) > 0))
      {
 	/* if we fit in the x axis, too, size already has a somewhat
 	 * meaningful value, so don't overwrite it with the starting
@@ -485,13 +512,8 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
 	       {
 		  int bottom, top;
 
-		  if (th < sh)
-		    bottom = 10;
-		  else if (th > sh)
-		    {
-		       bottom = 1;
-		       top = 10;
-		    }
+		  if (th < sh) bottom = 10;
+		  else if (th > sh) bottom = 1;
 		  else bottom = 0; /* XXX shut up GCC, th == sh is handled before! */
 
 		  top = size;
@@ -531,8 +553,9 @@ _edje_text_recalc_apply(Edje *ed, Edje_Real_Part *ep,
 	text = _edje_text_fit_x(ed, ep, params, text, font, size, sw, &free_text);
      }
 
+   str = eina_stringshare_add(text);
    if (ep->text.cache.out_str) eina_stringshare_del(ep->text.cache.out_str);
-   ep->text.cache.out_str = eina_stringshare_add(text);
+   ep->text.cache.out_str = str;
    ep->text.cache.in_w = sw;
    ep->text.cache.in_h = sh;
    ep->text.cache.out_size = size;
